package org.jtheque.films.persistence.od;

/*
 * This file is part of JTheque.
 *
 * JTheque is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License.
 *
 * JTheque is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JTheque.  If not, see <http://www.gnu.org/licenses/>.
 */

import org.jtheque.core.managers.Managers;
import org.jtheque.core.managers.resource.IResourceManager;
import org.jtheque.core.managers.resource.ImageType;
import org.jtheque.core.managers.properties.IPropertiesManager;
import org.jtheque.films.persistence.od.abstraction.Film;
import org.jtheque.films.persistence.od.temp.FilmTemporaryContext;
import org.jtheque.films.utils.Constants;
import org.jtheque.utils.EqualsUtils;

import javax.swing.Icon;

/**
 * Cette classe reprsente un film et toutes ses donnes.
 *
 * @author Baptiste Wicht
 */
public final class FilmImpl extends Film {
    private FilmImpl memento;
    private boolean mementoState;

    private final FilmTemporaryContext temporaryContext;

    /**
     * Construct a new FilmImpl.
     */
    public FilmImpl() {
        super();

        temporaryContext = new FilmTemporaryContext();
    }

    @Override
    public FilmTemporaryContext getTemporaryContext() {
        return temporaryContext;
    }

    @Override
    public String getAffichableText() {
        return getTitle();
    }

    @Override
    public String toString() {
        return getAffichableText();
    }

    @Override
    public int hashCode() {
        return Managers.getManager(IPropertiesManager.class).hashCode(this);
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }

        if(EqualsUtils.areObjectIncompatible(this, obj)){
            return false;
        }

        final Film other = (Film) obj;

        if(EqualsUtils.areNotEquals(getTitle(), other.getTitle())){
            return false;
        }

        if(EqualsUtils.areNotEquals(getActors(), other.getActors())){
            return false;
        }

        if(EqualsUtils.areNotEquals(getYear(), other.getYear())){
            return false;
        }

        if(EqualsUtils.areNotEquals(getKinds(), other.getKinds())){
            return false;
        }

        if(EqualsUtils.areNotEquals(getComment(), other.getComment())){
            return false;
        }

        if(EqualsUtils.areNotEquals(getDuration(), other.getDuration())){
            return false;
        }

        if(EqualsUtils.areNotEquals(getDuration(), other.getDuration())){
            return false;
        }

        if(EqualsUtils.areNotEquals(getTheLending(), other.getTheLending())){
            return false;
        }

        if(EqualsUtils.areNotEquals(getImage(), other.getImage())){
            return false;
        }

        if(EqualsUtils.areNotEquals(getTheLanguage(), other.getTheLanguage())){
            return false;
        }

        if(EqualsUtils.areNotEquals(getNote(), other.getNote())){
            return false;
        }

        if(EqualsUtils.areNotEquals(getTheRealizer(), other.getTheRealizer())){
            return false;
        }

        if(EqualsUtils.areNotEquals(getResume(), other.getResume())){
            return false;
        }

        return !EqualsUtils.areNotEquals(getTheType(), other.getTheType());

    }

    @Override
    public Icon getIcon() {
        return Managers.getManager(IResourceManager.class).getIcon(Constants.IMAGE_BASENAME, Constants.FILM_ICON, ImageType.PNG);
    }

    @Override
    public void saveToMemento() {
        mementoState = true;

        memento = Managers.getManager(IPropertiesManager.class).createMemento(this);

        if (memento == null) {
            mementoState = false;
        }
    }

    @Override
    public void restoreMemento() {
        if (mementoState) {
            Managers.getManager(IPropertiesManager.class).restoreMemento(this, memento);
        }
    }

    /**
     * Indicate if the film is in lending or not.
     *
     * @return <code>true</code> if the film has a lending else <code>false</code>
     */
    public boolean hasLending() {
        return getTheLending() != null;
    }

    /**
     * Indicate if the film is in lending or not.
     *
     * @return <code>true</code> if the film has a lending else <code>false</code>
     */
    public boolean hasType() {
        return getTheType() != null;
    }

    /**
     * Indicate if the film has kinds or not.
     *
     * @return <code>true</code> if the film has kinds else <code>false</code>
     */
    public boolean hasKinds() {
        return getKinds().isEmpty();
    }

    /**
     * Indicate if the film has actors or not.
     *
     * @return <code>true</code> if the film has actors else <code>false</code>
     */
    public boolean hasActors() {
        return getActors().isEmpty();
    }

    /**
     * Indicate if the film has a language or not.
     *
     * @return <code>true</code> if the film has a language else <code>false</code>
     */
    public boolean hasLanguage() {
        return getTheLanguage() != null;
    }

    /**
     * Indicate if the film has a realizer or not.
     *
     * @return <code>true</code> if the film has a realizer else <code>false</code>
     */
    public boolean hasRealizer() {
        return getTheRealizer() != null;
    }

    /**
     * Indicate if the film has a resume or not.
     *
     * @return <code>true</code> if the film has a resume else <code>false</code>
     */
    public boolean hasResume() {
        return getResume() != null && !getResume().isEmpty();
    }

    /**
     * Indicate if the film has a comment or not.
     *
     * @return <code>true</code> if the film has a comment else <code>false</code>
     */
    public boolean hasComment() {
        return getComment() != null && !getComment().isEmpty();
    }

    /**
     * Indicate if the film has an image or not.
     *
     * @return <code>true</code> if the film has an image else <code>false</code>
     */
    public boolean hasImage() {
        return getImage() != null && !getImage().isEmpty();
    }
}