package org.jtheque.films.services.impl.utils.file.backup;

/*
 * This file is part of JTheque.
 *
 * JTheque is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License.
 *
 * JTheque is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JTheque.  If not, see <http://www.gnu.org/licenses/>.
 */

import org.jtheque.core.managers.Managers;
import org.jtheque.core.managers.log.ILoggingManager;
import org.jtheque.core.managers.beans.IBeansManager;
import org.jtheque.core.managers.file.IFileManager;
import org.jtheque.core.managers.file.able.BackupWriter;
import org.jtheque.core.managers.file.able.BasicDataSource;
import org.jtheque.films.persistence.od.ActorImpl;
import org.jtheque.films.persistence.od.FilmImpl;
import org.jtheque.films.persistence.od.RealizerImpl;
import org.jtheque.films.services.able.IActorService;
import org.jtheque.films.services.able.IFilmsService;
import org.jtheque.films.services.able.IRealizersService;
import org.jtheque.primary.od.*;
import org.jtheque.primary.services.able.*;

import javax.annotation.Resource;
import java.io.DataOutputStream;
import java.io.IOException;

/**
 * A JTD Backup Writer.
 *
 * @author Baptiste Wicht
 */
public final class JTDBackupWriter implements BackupWriter {
    @Resource
    private IFilmsService filmsService;

    @Resource
    private IKindsService kindsService;

    @Resource
    private ITypesService typesService;

    @Resource
    private ILanguagesService languagesService;

    @Resource
    private IActorService actorService;

    @Resource
    private IRealizersService realizersService;

    @Resource
    private ILendingsService lendingsService;

    @Resource
    private IBorrowersService borrowersService;

    @Resource
    private ICountriesService countriesService;

    /**
     * Construct a new JTDBackupWriter.
     */
    public JTDBackupWriter() {
        super();

        Managers.getManager(IBeansManager.class).inject(this);
    }

    @Override
    public void populateDataSource(BasicDataSource datasource) {
    }

    @Override
    public void write(Object object) {
        DataOutputStream stream = (DataOutputStream) object;

        try {
            writeFilms(stream);

            stream.writeLong(IFileManager.JTCATEGORYSEPARATOR);

            writeActors(stream);

            stream.writeLong(IFileManager.JTCATEGORYSEPARATOR);

            writeRealizers(stream);

            stream.writeLong(IFileManager.JTCATEGORYSEPARATOR);

            writeLanguages(stream);

            stream.writeLong(IFileManager.JTCATEGORYSEPARATOR);

            writeKinds(stream);

            stream.writeLong(IFileManager.JTCATEGORYSEPARATOR);

            writeTypes(stream);

            stream.writeLong(IFileManager.JTCATEGORYSEPARATOR);

            writeCountries(stream);

            stream.writeLong(IFileManager.JTCATEGORYSEPARATOR);

            writeLendings(stream);

            stream.writeLong(IFileManager.JTCATEGORYSEPARATOR);

            writeBorrowers(stream);

            stream.writeLong(IFileManager.JTCATEGORYSEPARATOR);
        } catch (IOException e) {
            Managers.getManager(ILoggingManager.class).getLogger(getClass()).exception(e);
        }
    }

    /**
     * Write the borrowers to the stream.
     *
     * @param stream The stream to write in.
     * @throws IOException Thrown when an exception occurs durint the writing of data.
     */
    private void writeBorrowers(DataOutputStream stream) throws IOException {
        if (borrowersService.hasNoBorrowers()) {
            stream.writeInt(IFileManager.NOCONTENT);
        } else {
            stream.writeInt(IFileManager.CONTENT);

            boolean firstBorrower = true;
            for (BorrowerImpl borrower : borrowersService.getBorrowers()) {
                if (firstBorrower) {
                    firstBorrower = false;
                } else {
                    stream.writeLong(IFileManager.JTINTERNSEPARATOR);
                }

                stream.writeInt(borrower.getId());
                stream.writeUTF(Managers.getManager(IFileManager.class).formatUTFToWrite(borrower.getName()));
                stream.writeUTF(Managers.getManager(IFileManager.class).formatUTFToWrite(borrower.getFirstName()));
                stream.writeUTF(Managers.getManager(IFileManager.class).formatUTFToWrite(borrower.getEmail()));
            }
        }
    }

    /**
     * Write the lendings to the stream.
     *
     * @param stream The stream to write in.
     * @throws IOException Thrown when an exception occurs durint the writing of data.
     */
    private void writeLendings(DataOutputStream stream) throws IOException {
        if (lendingsService.hasNoLendings()) {
            stream.writeInt(IFileManager.NOCONTENT);
        } else {
            stream.writeInt(IFileManager.CONTENT);

            boolean firstLending = true;
            for (LendingImpl lending : lendingsService.getLendings()) {
                if (firstLending) {
                    firstLending = false;
                } else {
                    stream.writeLong(IFileManager.JTINTERNSEPARATOR);
                }

                stream.writeInt(lending.getId());
                stream.writeInt(lending.getDate().intValue());
                stream.writeInt(lending.getTheBorrower().getId());
                stream.writeInt(lending.getTheOther());
            }
        }
    }

    /**
     * Write the countries to the stream.
     *
     * @param stream The stream to write in.
     * @throws IOException Thrown when an exception occurs durint the writing of data.
     */
    private void writeCountries(DataOutputStream stream) throws IOException {
        if (countriesService.getCountries() == null || countriesService.getCountries().isEmpty()) {
            stream.writeInt(IFileManager.NOCONTENT);
        } else {
            stream.writeInt(IFileManager.CONTENT);

            boolean firstCountry = true;
            for (CountryImpl country : countriesService.getCountries()) {
                if (firstCountry) {
                    firstCountry = false;
                } else {
                    stream.writeLong(IFileManager.JTINTERNSEPARATOR);
                }

                stream.writeInt(country.getId());
                stream.writeUTF(Managers.getManager(IFileManager.class).formatUTFToWrite(country.getName()));
            }
        }
    }

    /**
     * Write the types to the stream.
     *
     * @param stream The stream to write in.
     * @throws IOException Thrown when an exception occurs durint the writing of data.
     */
    private void writeTypes(DataOutputStream stream) throws IOException {
        if (typesService.hasNoTypes()) {
            stream.writeInt(IFileManager.NOCONTENT);
        } else {
            stream.writeInt(IFileManager.CONTENT);

            boolean firstType = true;
            for (TypeImpl type : typesService.getTypes()) {
                if (firstType) {
                    firstType = false;
                } else {
                    stream.writeLong(IFileManager.JTINTERNSEPARATOR);
                }

                stream.writeInt(type.getId());
                stream.writeUTF(Managers.getManager(IFileManager.class).formatUTFToWrite(type.getName()));
            }
        }
    }

    /**
     * Write the kinds to the stream.
     *
     * @param stream The stream to write in.
     * @throws IOException Thrown when an exception occurs durint the writing of data.
     */
    private void writeKinds(DataOutputStream stream) throws IOException {
        if (kindsService.hasNoKinds()) {
            stream.writeInt(IFileManager.NOCONTENT);
        } else {
            stream.writeInt(IFileManager.CONTENT);

            boolean firstKind = true;
            for (KindImpl kind : kindsService.getKinds()) {
                if (firstKind) {
                    firstKind = false;
                } else {
                    stream.writeLong(IFileManager.JTINTERNSEPARATOR);
                }

                stream.writeInt(kind.getId());
                stream.writeUTF(Managers.getManager(IFileManager.class).formatUTFToWrite(kind.getName()));
            }
        }
    }

    /**
     * Write the languages to the stream.
     *
     * @param stream The stream to write in.
     * @throws IOException Thrown when an exception occurs durint the writing of data.
     */
    private void writeLanguages(DataOutputStream stream) throws IOException {
        if (languagesService.getLanguages() == null || languagesService.getLanguages().isEmpty()) {
            stream.writeInt(IFileManager.NOCONTENT);
        } else {
            stream.writeInt(IFileManager.CONTENT);

            boolean firstLanguage = true;
            for (LanguageImpl langue : languagesService.getLanguages()) {
                if (firstLanguage) {
                    firstLanguage = false;
                } else {
                    stream.writeLong(IFileManager.JTINTERNSEPARATOR);
                }

                stream.writeInt(langue.getId());
                stream.writeUTF(Managers.getManager(IFileManager.class).formatUTFToWrite(langue.getName()));
            }
        }
    }

    /**
     * Write the realizers to the stream.
     *
     * @param stream The stream to write in.
     * @throws IOException Thrown when an exception occurs durint the writing of data.
     */
    private void writeRealizers(DataOutputStream stream) throws IOException {
        if (realizersService.hasNoRealizers()) {
            stream.writeInt(IFileManager.NOCONTENT);
        } else {
            stream.writeInt(IFileManager.CONTENT);

            boolean firstRealizer = true;
            for (RealizerImpl realizer : realizersService.getRealizers()) {
                if (firstRealizer) {
                    firstRealizer = false;
                } else {
                    stream.writeLong(IFileManager.JTINTERNSEPARATOR);
                }

                stream.writeInt(realizer.getId());
                stream.writeUTF(Managers.getManager(IFileManager.class).formatUTFToWrite(realizer.getName()));
                stream.writeUTF(Managers.getManager(IFileManager.class).formatUTFToWrite(realizer.getFirstName()));
                stream.writeInt(realizer.getTheCountry().getId());
                stream.writeInt(realizer.getNote().getValue().intValue());
            }
        }
    }

    /**
     * Write the actors to the stream.
     *
     * @param stream The stream to write in.
     * @throws IOException Thrown when an exception occurs durint the writing of data.
     */
    private void writeActors(DataOutputStream stream) throws IOException {
        if (actorService.hasNoActor()) {
            stream.writeInt(IFileManager.NOCONTENT);
        } else {
            stream.writeInt(IFileManager.CONTENT);

            boolean firstActeur = true;
            for (ActorImpl actor : actorService.getActors()) {
                if (firstActeur) {
                    firstActeur = false;
                } else {
                    stream.writeLong(IFileManager.JTINTERNSEPARATOR);
                }

                stream.writeInt(actor.getId());
                stream.writeUTF(Managers.getManager(IFileManager.class).formatUTFToWrite(actor.getName()));
                stream.writeUTF(Managers.getManager(IFileManager.class).formatUTFToWrite(actor.getFirstName()));
                stream.writeInt(actor.getTheCountry().getId());
                stream.writeInt(actor.getNote().getValue().intValue());
            }
        }
    }

    /**
     * Write the films to the stream.
     *
     * @param stream The stream to write in.
     * @throws IOException Thrown when an exception occurs durint the writing of data.
     */
    private void writeFilms(DataOutputStream stream) throws IOException {
        if (filmsService.isNoFilms()) {
            stream.writeInt(IFileManager.NOCONTENT);
        } else {
            stream.writeInt(IFileManager.CONTENT);

            boolean firstFilm = true;
            for (FilmImpl film : filmsService.getFilms()) {
                if (firstFilm) {
                    firstFilm = false;
                } else {
                    stream.writeLong(IFileManager.JTINTERNSEPARATOR);
                }

                stream.writeInt(film.getId());
                stream.writeUTF(Managers.getManager(IFileManager.class).formatUTFToWrite(film.getTitle()));

                if (film.hasKinds()) {
                    stream.writeInt(IFileManager.NOCONTENT);
                } else {
                    stream.writeInt(IFileManager.CONTENT);

                    boolean firstKind = true;
                    for (KindImpl kind : film.getKinds()) {
                        if (firstKind) {
                            firstKind = false;
                        } else {
                            stream.writeLong(IFileManager.JTINTERNSEPARATOR);
                        }

                        stream.writeInt(kind.getId());
                    }

                    stream.writeLong(IFileManager.JTCATEGORYSEPARATOR);
                }

                if (film.hasType()) {
                    stream.writeInt(film.getTheType().getId());
                } else {
                    stream.writeInt(-1);
                }

                if (film.hasRealizer()) {
                    stream.writeInt(film.getTheRealizer().getId());
                } else {
                    stream.writeInt(-1);
                }

                if (film.hasLanguage()) {
                    stream.writeInt(film.getTheLanguage().getId());
                } else {
                    stream.writeInt(-1);
                }

                stream.writeInt(film.getYear());
                stream.writeInt(film.getDuration());
                stream.writeInt(film.getNote().getValue().intValue());
                stream.writeUTF(Managers.getManager(IFileManager.class).formatUTFToWrite(film.getResume()));
                stream.writeUTF(Managers.getManager(IFileManager.class).formatUTFToWrite(film.getComment()));

                if (film.hasActors()) {
                    stream.writeInt(IFileManager.NOCONTENT);
                } else {
                    stream.writeInt(IFileManager.CONTENT);

                    boolean firstActeur = true;
                    for (ActorImpl actor : film.getActors()) {
                        if (firstActeur) {
                            firstActeur = false;
                        } else {
                            stream.writeLong(IFileManager.JTINTERNSEPARATOR);
                        }

                        stream.writeInt(actor.getId());
                    }

                    stream.writeLong(IFileManager.JTCATEGORYSEPARATOR);
                }
            }
        }
    }
}