package org.jtheque.films.services.impl.utils.file.backup;

/*
 * This file is part of JTheque.
 *
 * JTheque is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License.
 *
 * JTheque is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JTheque.  If not, see <http://www.gnu.org/licenses/>.
 */

import org.jdom.Document;
import org.jdom.Element;
import org.jtheque.core.managers.Managers;
import org.jtheque.core.managers.beans.IBeansManager;
import org.jtheque.core.managers.file.able.BackupWriter;
import org.jtheque.core.managers.file.able.BasicDataSource;
import org.jtheque.films.persistence.od.ActorImpl;
import org.jtheque.films.persistence.od.FilmImpl;
import org.jtheque.films.persistence.od.RealizerImpl;
import org.jtheque.films.services.able.IActorService;
import org.jtheque.films.services.able.IFilmsService;
import org.jtheque.films.services.able.IRealizersService;
import org.jtheque.primary.od.*;
import org.jtheque.primary.services.able.*;

import javax.annotation.Resource;

/**
 * A XML Backup Writer.
 *
 * @author Baptiste Wicht
 */
public final class XMLBackupWriter implements BackupWriter {
    @Resource
    private ICountriesService countriesService;

    @Resource
    private ILendingsService lendingsService;

    @Resource
    private ITypesService typesService;

    @Resource
    private ILanguagesService languagesService;

    @Resource
    private IBorrowersService borrowersService;

    @Resource
    private IFilmsService filmsService;

    @Resource
    private IActorService actorService;

    @Resource
    private IRealizersService realizersService;

    @Resource
    private IKindsService kindsService;

    /**
     * Construct a new XMLBackupWriter.
     */
    public XMLBackupWriter() {
        super();

        Managers.getManager(IBeansManager.class).inject(this);
    }

    @Override
    public void write(Object object) {
        Document doc = (Document) object;

        writeFilms(doc);
        writeActors(doc);
        writeRealizers(doc);
        writeKinds(doc);
        writeTypes(doc);
        writeLanguages(doc);
        writeCountries(doc);
        writeBorrowers(doc);
        writeLendings(doc);
    }

    /**
     * Write all the films in the Document.
     *
     * @param doc The document in which add the header.
     */
    private void writeFilms(Document doc) {
        Element films = new Element("films");

        for (FilmImpl film : filmsService.getFilms()) {
            Element filmElement = new Element("film");

            Element id = new Element("id");
            id.setText(Integer.toString(film.getId()));
            filmElement.addContent(id);

            Element title = new Element("title");
            title.setText(film.getTitle());
            filmElement.addContent(title);

            Element kinds = new Element("kinds");

            for (ActorImpl actor : film.getActors()) {
                Element actorElement = new Element("kind");
                actorElement.setText(Integer.toString(actor.getId()));
                kinds.addContent(actorElement);
            }

            filmElement.addContent(kinds);

            Element type = new Element("type");

            if (film.hasRealizer()) {
                type.setText(Integer.toString(film.getTheType().getId()));
            } else {
                type.setText("-2");
            }

            filmElement.addContent(type);

            Element realizer = new Element("realizer");

            if (film.hasRealizer()) {
                realizer.setText(Integer.toString(film.getTheRealizer().getId()));
            } else {
                realizer.setText("-2");
            }

            filmElement.addContent(realizer);

            Element language = new Element("language");

            if (film.hasRealizer()) {
                language.setText(Integer.toString(film.getTheLanguage().getId()));
            } else {
                language.setText("-2");
            }

            filmElement.addContent(language);

            Element year = new Element("year");
            year.setText(Integer.toString(film.getYear()));
            filmElement.addContent(year);

            Element duration = new Element("duration");
            duration.setText(Integer.toString(film.getDuration()));
            filmElement.addContent(duration);

            Element note = new Element("note");
            note.setText(Integer.toString(film.getNote().getValue().intValue()));
            filmElement.addContent(note);

            Element resume = new Element("resume");
            resume.setText(film.getResume());
            filmElement.addContent(resume);

            Element comment = new Element("comment");
            comment.setText(film.getComment());
            filmElement.addContent(comment);

            Element actors = new Element("actors");

            for (ActorImpl actor : film.getActors()) {
                Element actorElement = new Element("actor");
                actorElement.setText(Integer.toString(actor.getId()));
                actors.addContent(actorElement);
            }

            filmElement.addContent(actors);

            films.addContent(filmElement);
        }

        doc.getRootElement().addContent(films);
    }

    /**
     * Write all the actors in the Document.
     *
     * @param doc The document in which we write.
     */
    private void writeActors(Document doc) {
        Element actors = new Element("actors");

        for (ActorImpl actor : actorService.getActors()) {
            Element actorElement = new Element("actor");

            Element id = new Element("id");
            id.setText(Integer.toString(actor.getId()));
            actorElement.addContent(id);

            Element name = new Element("name");
            name.setText(actor.getName());
            actorElement.addContent(name);

            Element firstname = new Element("firstname");
            firstname.setText(actor.getFirstName());
            actorElement.addContent(firstname);

            Element country = new Element("country");
            country.setText(Integer.toString(actor.getTheCountry().getId()));
            actorElement.addContent(country);

            Element note = new Element("note");
            note.setText(Integer.toString(actor.getNote().getValue().intValue()));
            actorElement.addContent(note);

            actors.addContent(actorElement);
        }

        doc.getRootElement().addContent(actors);
    }

    /**
     * Write all the realizers in the Document.
     *
     * @param doc The document in which we write.
     */
    private void writeRealizers(Document doc) {
        Element realizers = new Element("realizers");

        for (RealizerImpl realizer : realizersService.getRealizers()) {
            Element realizerElement = new Element("realizer");

            Element id = new Element("id");
            id.setText(Integer.toString(realizer.getId()));
            realizerElement.addContent(id);

            Element name = new Element("name");
            name.setText(realizer.getName());
            realizerElement.addContent(name);

            Element firstname = new Element("firstname");
            firstname.setText(realizer.getFirstName());
            realizerElement.addContent(firstname);

            Element country = new Element("country");
            country.setText(Integer.toString(realizer.getTheCountry().getId()));
            realizerElement.addContent(country);

            Element note = new Element("note");
            note.setText(Integer.toString(realizer.getNote().getValue().intValue()));
            realizerElement.addContent(note);

            realizers.addContent(realizerElement);
        }

        doc.getRootElement().addContent(realizers);
    }

    /**
     * Write all the languages to the Document.
     *
     * @param doc The Document in which we write
     */
    private void writeLanguages(Document doc) {
        Element languages = new Element("languages");

        for (LanguageImpl language : languagesService.getLanguages()) {
            Element languageElement = new Element("language");

            Element id = new Element("id");
            id.setText(Integer.toString(language.getId()));
            languageElement.addContent(id);

            Element name = new Element("name");
            name.setText(language.getName());
            languageElement.addContent(name);

            languages.addContent(languageElement);
        }

        doc.getRootElement().addContent(languages);
    }

    /**
     * Write all the kinds to the Document.
     *
     * @param doc The Document in which we write
     */
    private void writeKinds(Document doc) {
        Element kinds = new Element("kinds");

        for (KindImpl kind : kindsService.getKinds()) {
            Element kindElement = new Element("kind");

            Element id = new Element("id");
            id.setText(Integer.toString(kind.getId()));
            kindElement.addContent(id);

            Element name = new Element("name");
            name.setText(kind.getName());
            kindElement.addContent(name);

            kinds.addContent(kindElement);
        }

        doc.getRootElement().addContent(kinds);
    }

    /**
     * Write all the kinds to the Document.
     *
     * @param doc The Document in which we write
     */
    private void writeTypes(Document doc) {
        Element types = new Element("types");

        for (TypeImpl type : typesService.getTypes()) {
            Element typeElement = new Element("type");

            Element id = new Element("id");
            id.setText(Integer.toString(type.getId()));
            typeElement.addContent(id);

            Element name = new Element("name");
            name.setText(type.getName());
            typeElement.addContent(name);

            types.addContent(typeElement);
        }

        doc.getRootElement().addContent(types);
    }

    /**
     * Write all the countries to the Document.
     *
     * @param doc The Document in which we write
     */
    private void writeCountries(Document doc) {
        Element countries = new Element("countries");

        for (CountryImpl country : countriesService.getCountries()) {
            Element typeElement = new Element("country");

            Element id = new Element("id");
            id.setText(Integer.toString(country.getId()));
            typeElement.addContent(id);

            Element name = new Element("name");
            name.setText(country.getName());
            typeElement.addContent(name);

            countries.addContent(typeElement);
        }

        doc.getRootElement().addContent(countries);
    }

    /**
     * Write all borrowers in the Document.
     *
     * @param doc The document in which we write.
     */
    private void writeBorrowers(Document doc) {
        Element borrowers = new Element("borrowers");

        for (BorrowerImpl borrower : borrowersService.getBorrowers()) {
            Element borrowerElement = new Element("borrower");

            Element id = new Element("id");
            id.setText(Integer.toString(borrower.getId()));
            borrowerElement.addContent(id);

            Element name = new Element("name");
            name.setText(borrower.getName());
            borrowerElement.addContent(name);

            Element firstName = new Element("firstname");
            firstName.setText(borrower.getFirstName());
            borrowerElement.addContent(firstName);

            Element email = new Element("email");
            email.setText(borrower.getEmail());
            borrowerElement.addContent(email);

            borrowers.addContent(borrowerElement);
        }

        doc.getRootElement().addContent(borrowers);
    }

    /**
     * Write all lendings in the Document.
     *
     * @param doc The document in which we write.
     */
    private void writeLendings(Document doc) {
        Element lendings = new Element("lendings");

        for (LendingImpl lending : lendingsService.getLendings()) {
            Element lendingElement = new Element("lending");

            Element id = new Element("id");
            id.setText(Integer.toString(lending.getId()));
            lendingElement.addContent(id);

            Element date = new Element("date");
            date.setText(Integer.toString(lending.getDate().intValue()));
            lendingElement.addContent(date);

            Element borrower = new Element("borrower");
            borrower.setText(Integer.toString(lending.getTheBorrower().getId()));
            lendingElement.addContent(borrower);

            Element film = new Element("film");
            film.setText(Integer.toString(lending.getTheOther()));
            lendingElement.addContent(film);

            lendings.addContent(lendingElement);
        }

        doc.getRootElement().addContent(lendings);
    }

    @Override
    public void populateDataSource(BasicDataSource datasource) {
    }
}