package org.jtheque.films.services.impl.utils.file.exports.datasources;

import net.sf.jasperreports.engine.JRDataSource;
import net.sf.jasperreports.engine.JRException;
import net.sf.jasperreports.engine.JRField;
import org.jtheque.films.persistence.od.ActorImpl;
import org.jtheque.films.persistence.od.FilmImpl;
import org.jtheque.primary.od.KindImpl;

import java.util.ArrayList;
import java.util.List;

/*
 * This file is part of JTheque.
 *
 * JTheque is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License.
 *
 * JTheque is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JTheque.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * A films jasper data source.
 *
 * @author Baptiste Wicht
 */
public final class FilmsDatasource implements JRDataSource {
    private final List<FilmImpl> films;

    private int index = -1;

    /**
     * Construct a new FilmsDatasource for a list of films.
     *
     * @param films The films to provide in the data source.
     */
    public FilmsDatasource(List<FilmImpl> films) {
        super();

        this.films = new ArrayList<FilmImpl>(films);
    }

    /**
     * Construct a new FilmsDatasource for a film.
     *
     * @param film The film to provide in the data source.
     */
    public FilmsDatasource(FilmImpl film) {
        super();

        films = new ArrayList<FilmImpl>(1);
        films.add(film);
    }

    @Override
    public boolean next() throws JRException {
        index++;

        return index < films.size();
    }

    @Override
    public Object getFieldValue(JRField field) throws JRException {
        FilmImpl film = films.get(index);

        String fieldName = field.getName();

        Object value = null;

        if ("title".equals(fieldName)) {
            value = film.getTitle();
        } else if ("kinds".equals(fieldName)) {
            StringBuilder builder = new StringBuilder(50);

            if (film.hasKinds()) {
                boolean firstKind = true;
                for (KindImpl kind : film.getKinds()) {
                    if (firstKind) {
                        builder.append(kind.getAffichableText());
                        firstKind = false;
                    } else {
                        builder.append(", ").append(kind.getAffichableText());
                    }
                }
            }

            value = builder.toString();
        } else if ("type".equals(fieldName)) {
            if (film.hasType()) {
                value = film.getTheType().getAffichableText();
            }
        } else if ("realizer".equals(fieldName)) {
            if (film.hasRealizer()) {
                value = film.getTheRealizer().getAffichableText();
            }
        } else if ("year".equals(fieldName)) {
            value = Integer.toString(film.getYear());
        } else if ("duration".equals(fieldName)) {
            int hours = film.getDuration() / 60;
            int minutes = film.getDuration() % 60;

            value = hours + "h" + minutes;
        } else if ("language".equals(fieldName)) {
            value = film.getTheLanguage().getAffichableText();
        } else if ("actors".equals(fieldName)) {
            StringBuilder builder = new StringBuilder(200);

            boolean firstActeur = true;
            for (ActorImpl actor : film.getActors()) {
                if (firstActeur) {
                    builder.append(actor.getAffichableText());
                    firstActeur = false;
                } else {
                    builder.append(", ").append(actor.getAffichableText());
                }
            }

            value = builder.toString();
        } else if ("note".equals(fieldName)) {
            value = film.getNote().getInternationalizedText();
        } else if ("image".equals(fieldName)) {
            value = film.getImage();
        } else if ("resume".equals(fieldName)) {
            value = film.getResume();
        }

        return value;
    }
}