package org.jtheque.films.services.impl.utils.file.imports;

/*
 * This file is part of JTheque.
 *
 * JTheque is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License.
 *
 * JTheque is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JTheque.  If not, see <http://www.gnu.org/licenses/>.
 */

import org.jtheque.core.managers.Managers;
import org.jtheque.core.managers.beans.IBeansManager;
import org.jtheque.core.utils.db.DaoNotes;
import org.jtheque.core.utils.db.DaoNotes.NoteType;
import org.jtheque.films.persistence.od.ActorImpl;
import org.jtheque.films.persistence.od.FilmImpl;
import org.jtheque.films.persistence.od.RealizerImpl;
import org.jtheque.films.services.able.IActorService;
import org.jtheque.films.services.able.IFilmsService;
import org.jtheque.films.services.able.IRealizersService;
import org.jtheque.films.services.impl.utils.DataUtils;
import org.jtheque.primary.od.*;
import org.jtheque.primary.services.able.*;

import javax.annotation.Resource;
import java.util.HashSet;
import java.util.Set;

/**
 * Utils for the importer.
 *
 * @author Baptiste Wicht
 */
public final class ImporterUtils {
    @Resource
    private static ICountriesService countriesService;

    @Resource
    private static ILendingsService lendingsService;

    @Resource
    private static ITypesService typesService;

    @Resource
    private static ILanguagesService languagesService;

    @Resource
    private static IBorrowersService borrowersService;

    @Resource
    private static IFilmsService filmsService;

    @Resource
    private static IActorService actorService;

    @Resource
    private static IRealizersService realizersService;

    @Resource
    private static IKindsService kindsService;

    private static final DaoNotes daoNotes = DaoNotes.getINSTANCE();

    /**
     * Construct a new ImporterUtils.
     */
    private ImporterUtils() {
        super();

        Managers.getManager(IBeansManager.class).inject(this);
    }

    /**
     * Persist the data of an import.
     *
     * @param films     The films.
     * @param actors    The actors.
     * @param realizers The realizers.
     * @param kinds     The kinds.
     * @param types     The types.
     * @param languages The languages.
     * @param countries The countries.
     * @param borrowers The borrowers.
     * @param lendings  The lendings.
     */
    public static void persistDataOfImport(Iterable<FilmImpl> films, Iterable<ActorImpl> actors,
                                           Iterable<RealizerImpl> realizers, Iterable<KindImpl> kinds,
                                           Iterable<TypeImpl> types, Iterable<LanguageImpl> languages,
                                           Iterable<CountryImpl> countries,
                                           Iterable<BorrowerImpl> borrowers, Iterable<LendingImpl> lendings) {

        countriesService.createAll(countries);

        if (borrowers != null) {
            for (BorrowerImpl borrower : borrowers) {
                borrowersService.create(borrower);
            }
        }

        languagesService.createAll(languages);
        typesService.createAll(types);
        kindsService.createAll(kinds);

        for (RealizerImpl realizer : realizers) {
            realizer.setTheCountry(DataUtils.getDataByTemporaryId(realizer.getTemporaryContext().getCountry(), countries));
            realizer.setNote(daoNotes.getNote(NoteType.getEnum(realizer.getTemporaryContext().getIntNote())));

            realizersService.create(realizer);
        }

        for (ActorImpl actor : actors) {
            actor.setTheCountry(DataUtils.getDataByTemporaryId(actor.getTemporaryContext().getCountry(), countries));
            actor.setNote(daoNotes.getNote(NoteType.getEnum(actor.getTemporaryContext().getIntNote())));

            actorService.create(actor);
        }

        for (FilmImpl film : films) {
            film.setTheLanguage(DataUtils.getDataByTemporaryId(film.getTemporaryContext().getLanguage(), languages));
            film.setTheType(DataUtils.getDataByTemporaryId(film.getTemporaryContext().getType(), types));
            film.setTheRealizer(DataUtils.getDataByTemporaryId(film.getTemporaryContext().getRealizer(), realizers));

            Set<ActorImpl> actorsOfFilm = new HashSet<ActorImpl>(film.getTemporaryContext().getActors().size());

            for (Integer i : film.getTemporaryContext().getActors()) {
                ActorImpl actor = DataUtils.getDataByTemporaryId(i, actors);

                if (actor != null) {
                    actorsOfFilm.add(actor);
                }
            }

            film.setActors(actorsOfFilm);

            Set<KindImpl> kindsOfFilm = new HashSet<KindImpl>(film.getTemporaryContext().getKinds().size());

            for (Integer i : film.getTemporaryContext().getKinds()) {
                KindImpl kind = DataUtils.getDataByTemporaryId(i, kinds);

                if (kind != null) {
                    kindsOfFilm.add(kind);
                }
            }

            film.setKinds(kindsOfFilm);

            filmsService.create(film);
        }

        if (lendings != null) {
            for (LendingImpl lending : lendings) {
                lending.setTheBorrower(DataUtils.getDataByTemporaryId(lending.getTemporaryContext().getBorrower(), borrowers));
                lending.setTheOther(DataUtils.getDataByTemporaryId(lending.getTemporaryContext().getFilm(), films).getId());

                lendingsService.create(lending);
            }
        }
    }
}