package org.jtheque.films.services.impl.utils.file.imports;

/*
 * This file is part of JTheque.
 *
 * JTheque is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License.
 *
 * JTheque is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JTheque.  If not, see <http://www.gnu.org/licenses/>.
 */

import org.jtheque.core.managers.Managers;
import org.jtheque.core.managers.language.ILanguageManager;
import org.jtheque.core.managers.beans.IBeansManager;
import org.jtheque.core.utils.db.DaoNotes;
import org.jtheque.core.utils.db.DaoNotes.NoteType;
import org.jtheque.core.utils.file.XMLException;
import org.jtheque.core.utils.file.XMLReader;
import org.jtheque.films.persistence.od.ActorImpl;
import org.jtheque.films.persistence.od.FilmImpl;
import org.jtheque.films.persistence.od.RealizerImpl;
import org.jtheque.films.services.able.IActorService;
import org.jtheque.films.services.able.IFilmsService;
import org.jtheque.films.services.able.INotesService;
import org.jtheque.films.services.able.IRealizersService;
import org.jtheque.films.utils.Constants;
import org.jtheque.films.utils.Constants.Files.FileType;
import org.jtheque.primary.od.CountryImpl;
import org.jtheque.primary.od.LanguageImpl;
import org.jtheque.primary.od.TypeImpl;
import org.jtheque.primary.od.KindImpl;
import org.jtheque.primary.services.able.ICountriesService;
import org.jtheque.primary.services.able.ILanguagesService;
import org.jtheque.primary.services.able.ITypesService;
import org.jtheque.primary.services.able.IKindsService;
import org.jtheque.utils.io.FileUtils;
import org.jtheque.utils.io.FileException;

import javax.annotation.Resource;

/**
 * Cette classe permet d'importer des fichiers XML dans la base de donnes.
 *
 * @author Baptiste Wicht
 */
public final class XMLImporter extends AbstractImporter {
    private final ILanguageManager resources = Managers.getManager(ILanguageManager.class);

    @Resource
    private ICountriesService countriesService;

    @Resource
    private ITypesService typesService;

    @Resource
    private ILanguagesService languagesService;

    @Resource
    private IFilmsService filmsService;

    @Resource
    private IActorService actorService;

    @Resource
    private IRealizersService realizersService;

    @Resource
    private INotesService notesService;

    private final DaoNotes daoNotes = DaoNotes.getINSTANCE();

    @Resource
    private IKindsService kindsService;

    /**
     * Construct a new XMLImporter.
     */
    public XMLImporter() {
        super();

        Managers.getManager(IBeansManager.class).inject(this);
    }

    @Override
    public boolean canImportFrom(FileType fileType) {
        return fileType == FileType.XML;
    }

    @Override
    public void importFrom(String filePath) throws FileException {
        XMLReader reader = new XMLReader();

        try {
            reader.openFile(filePath);

            int version = reader.readInt("./header/file-version", reader.getRootElement());

            if (version != Constants.Files.Versions.XML.FIRST || version != Constants.Files.Versions.XML.SECOND) {
                throw new FileException(resources.getMessage("errors.file.unsupportedversion"));
            }

            for (Object currentNode : reader.getNodes("//film", reader.getRootElement())) {
                FilmImpl film = new FilmImpl();

                film.setTitle(reader.readString("./title", currentNode));
                film.setDuration(reader.readInt("./duration", currentNode));
                film.setYear(reader.readInt("./year", currentNode));
                film.setNote(daoNotes.getNote(NoteType.getEnum(reader.readInt("./note", currentNode))));
                film.setComment(reader.readString("./comment", currentNode));
                film.setResume(reader.readString("./resume", currentNode));

                importLanguage(reader, currentNode, film);
                importType(reader, currentNode, film);
                importRealizer(reader, currentNode, film);
                importKinds(reader, version, currentNode, film);
                importActors(reader, currentNode, film);

                filmsService.create(film);
            }
        } catch (XMLException e) {
            throw new FileException(resources.getMessage("errors.file.ioexception"), e);
        } finally {
            FileUtils.close(reader);
        }
    }

    private void importLanguage(XMLReader reader, Object currentNode, FilmImpl film) throws XMLException {
        LanguageImpl language = new LanguageImpl();
        language.setName(reader.readString("./language", currentNode));

        if (languagesService.exist(language)) {
            language.setId(languagesService.getLanguage(language.getName()).getId());
        } else {
            languagesService.create(language);
        }

        film.setTheLanguage(language);
    }

    private void importType(XMLReader reader, Object currentNode, FilmImpl film) throws XMLException {
        TypeImpl type = new TypeImpl();
        type.setName(reader.readString("./type", currentNode));

        if (typesService.exists(type)) {
            type.setId(typesService.getType(type.getName()).getId());
        } else {
            typesService.create(type);
        }

        film.setTheType(type);
    }

    private void importRealizer(XMLReader reader, Object currentNode, FilmImpl film) throws XMLException {
        Object realizerNode = reader.getNode("./realizer", currentNode);

        if (realizerNode != null) {
            RealizerImpl realizer = new RealizerImpl();
            realizer.setName(reader.readString("./name", realizerNode));
            realizer.setFirstName(reader.readString("./firstname", realizerNode));
            realizer.setNote(notesService.getDefaultNote());

            CountryImpl countryRealizer = new CountryImpl();
            countryRealizer.setName(reader.readString("./country", realizerNode));

            if (countriesService.exist(countryRealizer)) {
                realizer.setTheCountry(countriesService.getCountry(countryRealizer.getAffichableText()));
            } else {
                countriesService.create(countryRealizer);
                realizer.setTheCountry(countryRealizer);
            }

            if (realizersService.exists(realizer)) {
                film.setTheRealizer(realizersService.getRealizer(realizer.getFirstName(), realizer.getName()));
            } else {
                realizersService.create(realizer);
                film.setTheRealizer(realizer);
            }
        }
    }

    private void importKinds(XMLReader reader, int version, Object currentNode, FilmImpl film) throws XMLException {
        if (version == Constants.Files.Versions.XML.FIRST) {
            String name = reader.readString("./kind", currentNode);

            if (kindsService.exists(name)) {
                film.getKinds().add(kindsService.getKind(name));
            } else {
                KindImpl kind = new KindImpl();
                kind.setName(name);

                kindsService.create(kind);
                film.getKinds().add(kind);
            }
        } else {
            for (Object kindNode : reader.getNodes("//kinds/kind", currentNode)) {
                String name = reader.readString("./name", kindNode);

                if (kindsService.exists(name)) {
                    film.getKinds().add(kindsService.getKind(name));
                } else {
                    KindImpl kind = new KindImpl();
                    kind.setName(name);

                    kindsService.create(kind);
                    film.getKinds().add(kind);
                }
            }
        }
    }

    private void importActors(XMLReader reader, Object currentNode, FilmImpl film) throws XMLException {
        for (Object actorNode : reader.getNodes("//actors/actor", currentNode)) {
            ActorImpl actor = new ActorImpl();
            actor.setName(reader.readString("./name", actorNode));
            actor.setFirstName(reader.readString("./firstname", actorNode));
            actor.setNote(notesService.getDefaultNote());

            CountryImpl countryActor = new CountryImpl();
            countryActor.setName(reader.readString("./country", actorNode));

            if (countriesService.exist(countryActor)) {
                actor.setTheCountry(countriesService.getCountry(countryActor.getAffichableText()));
            } else {
                countriesService.create(countryActor);
                actor.setTheCountry(countryActor);
            }

            if (actorService.exist(actor)) {
                film.getActors().add(actorService.getActor(actor.getFirstName(), actor.getName()));
            } else {
                actorService.create(actor);
                film.getActors().add(actor);
            }
        }
    }
}