package org.jtheque.films.services.impl.utils.file.jt.reader;

/*
 * This file is part of JTheque.
 *
 * JTheque is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License.
 *
 * JTheque is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JTheque.  If not, see <http://www.gnu.org/licenses/>.
 */

import org.jtheque.core.managers.Managers;
import org.jtheque.core.managers.language.ILanguageManager;
import org.jtheque.core.managers.file.IFileManager;
import org.jtheque.utils.bean.Version;
import org.jtheque.utils.io.FileUtils;
import org.jtheque.utils.io.FileException;
import org.jtheque.core.utils.file.jt.AbstractJTFileHeader;
import org.jtheque.core.utils.file.jt.JTFileReader;
import org.jtheque.films.persistence.od.ActorImpl;
import org.jtheque.films.persistence.od.FilmImpl;
import org.jtheque.films.persistence.od.RealizerImpl;
import org.jtheque.films.services.impl.utils.file.jt.JTFFile;
import org.jtheque.films.utils.Constants;
import org.jtheque.primary.od.CountryImpl;
import org.jtheque.primary.od.LanguageImpl;
import org.jtheque.primary.od.KindImpl;
import org.jtheque.primary.od.TypeImpl;

import java.io.DataInputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

/**
 * Cette classe reprsente un reader de fichier JTF.
 *
 * @author Baptiste Wicht
 */
public final class JTFFileReader extends JTFileReader {
    private boolean correctSeparators;
    
    @Override
    public JTFFile readFile(DataInputStream stream) throws FileException {
        JTFFile file = new JTFFile();
        
        correctSeparators = true;
        
        try {
            AbstractJTFileHeader header = file.getHeader();

            header.setKey(stream.readUTF());
            header.setVersionJTheque(new Version(stream.readUTF()));
            header.setFileVersion(stream.readInt());
            header.setDate(stream.readInt());

            if (stream.readLong() != Constants.Files.JT.JTCATEGORYSEPARATOR) {
                correctSeparators = false;
            }

            FilmImpl film = new FilmImpl();

            film.setTitle(Managers.getManager(IFileManager.class).formatUTFToRead(stream.readUTF()));
            film.setYear(stream.readInt());
            film.setDuration(stream.readInt());
            film.getTemporaryContext().setTemporaryIntNote(stream.readInt());
            film.setResume(Managers.getManager(IFileManager.class).formatUTFToRead(stream.readUTF()));
            film.setComment(Managers.getManager(IFileManager.class).formatUTFToRead(stream.readUTF()));

            file.setFilm(film);

            if (stream.readLong() != Constants.Files.JT.JTCATEGORYSEPARATOR) {
                correctSeparators = false;
            }
            
            readActors(stream, file);
            readRealizer(stream, file);
            readLanguage(stream, file);
            readKinds(stream, file, header);
            readType(stream, file);
            readCountries(stream, file);
        } catch (IOException e) {
            throw new FileException(Managers.getManager(ILanguageManager.class).getMessage("errors.file.ioexception"), e);
        } finally {
            FileUtils.close(stream);
        }

        file.setCorrectSeparators(correctSeparators);

        return file;
    }

    private void readActors(DataInputStream stream, JTFFile file) throws IOException {
        if (stream.readInt() == Constants.Files.JT.CONTENT) {
            boolean endOfActeursList = false;
            Set<ActorImpl> acteurs = new HashSet<ActorImpl>(12);

            while (!endOfActeursList) {
                ActorImpl acteur = new ActorImpl();

                acteur.setName(Managers.getManager(IFileManager.class).formatUTFToRead(stream.readUTF()));
                acteur.setFirstName(Managers.getManager(IFileManager.class).formatUTFToRead(stream.readUTF()));
                acteur.getTemporaryContext().setCountry(stream.readInt());
                acteur.getTemporaryContext().setIntNote(stream.readInt());

                acteurs.add(acteur);

                long separator = stream.readLong();

                if (separator == Constants.Files.JT.JTCATEGORYSEPARATOR) {
                    endOfActeursList = true;
                } else if (separator != Constants.Files.JT.JTINTERNSEPARATOR) {
                    correctSeparators = false;
                }
            }

            file.setActeurs(acteurs);
        } else {
            file.setActeurs(Collections.<ActorImpl>emptySet());

            if (stream.readLong() != Constants.Files.JT.JTCATEGORYSEPARATOR) {
                correctSeparators = false;
            }
        }
    }

    private void readRealizer(DataInputStream stream, JTFFile file) throws IOException {
        //Si y a un ralisateur
        if (stream.readInt() == Constants.Files.JT.CONTENT) {
            RealizerImpl realizer = new RealizerImpl();

            realizer.setName(Managers.getManager(IFileManager.class).formatUTFToRead(stream.readUTF()));
            realizer.setFirstName(Managers.getManager(IFileManager.class).formatUTFToRead(stream.readUTF()));
            realizer.getTemporaryContext().setCountry(stream.readInt());
            realizer.getTemporaryContext().setIntNote(stream.readInt());

            file.setRealisateur(realizer);
        }

        if (stream.readLong() != Constants.Files.JT.JTCATEGORYSEPARATOR) {
            correctSeparators = false;
        }
    }

    private void readLanguage(DataInputStream stream, JTFFile file) throws IOException {
        if (stream.readInt() == Constants.Files.JT.CONTENT) {
            LanguageImpl langue = new LanguageImpl();

            langue.setName(Managers.getManager(IFileManager.class).formatUTFToRead(stream.readUTF()));

            file.setLangue(langue);
        }

        if (stream.readLong() != Constants.Files.JT.JTCATEGORYSEPARATOR) {
            correctSeparators = false;
        }
    }

    private void readKinds(DataInputStream stream, JTFFile file, AbstractJTFileHeader header) throws IOException {
        if (header.getFileVersion() == Constants.Files.Versions.JTF.FIRST) {
            //Si y a un genre
            if (stream.readInt() == Constants.Files.JT.CONTENT) {
                KindImpl kind = new KindImpl();

                kind.setName(Managers.getManager(IFileManager.class).formatUTFToRead(stream.readUTF()));

                Set<KindImpl> kinds = new HashSet<KindImpl>(1);

                kinds.add(kind);

                file.setKinds(kinds);
            }

            if (stream.readLong() != Constants.Files.JT.JTCATEGORYSEPARATOR) {
                correctSeparators = false;
            }
        } else if (header.getFileVersion() == Constants.Files.Versions.JTF.SECOND) {
            //Si y a des genres
            if (stream.readInt() == Constants.Files.JT.CONTENT) {
                boolean endOfKindsList = false;
                Set<KindImpl> kinds = new HashSet<KindImpl>(5);

                while (!endOfKindsList) {
                    KindImpl kind = new KindImpl();

                    kind.setName(Managers.getManager(IFileManager.class).formatUTFToRead(stream.readUTF()));

                    kinds.add(kind);

                    long separator = stream.readLong();

                    if (separator == Constants.Files.JT.JTCATEGORYSEPARATOR) {
                        endOfKindsList = true;
                    } else if (separator != Constants.Files.JT.JTINTERNSEPARATOR) {
                        correctSeparators = false;
                    }
                }

                file.setKinds(kinds);
            } else {
                file.setKinds(Collections.<KindImpl>emptySet());

                if (stream.readLong() != Constants.Files.JT.JTCATEGORYSEPARATOR) {
                    correctSeparators = false;
                }
            }
        }
    }

    private void readType(DataInputStream stream, JTFFile file) throws IOException {
        //Si y a un type
        if (stream.readInt() == Constants.Files.JT.CONTENT) {
            TypeImpl type = new TypeImpl();

            type.setName(Managers.getManager(IFileManager.class).formatUTFToRead(stream.readUTF()));

            file.setType(type);
        }

        if (stream.readLong() != Constants.Files.JT.JTCATEGORYSEPARATOR) {
            correctSeparators = false;
        }
    }

    private void readCountries(DataInputStream stream, JTFFile file) throws IOException {
        //Si y a des pays
        if (stream.readInt() == Constants.Files.JT.CONTENT) {
            boolean endOfCountries = false;
            List<CountryImpl> countries = new ArrayList<CountryImpl>(12);

            while (!endOfCountries) {
                CountryImpl country = new CountryImpl();

                country.getTemporaryContext().setId(stream.readInt());
                country.setName(Managers.getManager(IFileManager.class).formatUTFToRead(stream.readUTF()));

                countries.add(country);

                long separator = stream.readLong();

                if (separator == Constants.Files.JT.JTCATEGORYSEPARATOR) {
                    endOfCountries = true;
                } else if (separator != Constants.Files.JT.JTINTERNSEPARATOR) {
                    correctSeparators = false;
                }
            }

            file.setCountries(countries);
        } else {
            file.setCountries(Collections.<CountryImpl>emptyList());

            if (stream.readLong() != Constants.Files.JT.JTCATEGORYSEPARATOR) {
                correctSeparators = false;
            }
        }
    }
}