package org.jtheque.films.services.impl.utils.file.restore;

/*
 * This file is part of JTheque.
 *
 * JTheque is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License.
 *
 * JTheque is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JTheque.  If not, see <http://www.gnu.org/licenses/>.
 */

import org.jtheque.core.managers.Managers;
import org.jtheque.core.managers.log.ILoggingManager;
import org.jtheque.core.managers.beans.IBeansManager;
import org.jtheque.core.managers.file.able.BackupReader;
import org.jtheque.films.persistence.od.ActorImpl;
import org.jtheque.films.persistence.od.FilmImpl;
import org.jtheque.films.persistence.od.RealizerImpl;
import org.jtheque.films.services.able.INotesService;
import org.jtheque.films.services.impl.utils.file.imports.ImporterUtils;
import org.jtheque.primary.od.CountryImpl;
import org.jtheque.primary.od.LanguageImpl;
import org.jtheque.primary.od.KindImpl;
import org.jtheque.primary.od.TypeImpl;
import org.jtheque.utils.DatabaseException;

import javax.annotation.Resource;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.List;

/**
 * A reader for a database V4 backup.
 *
 * @author Baptiste Wicht
 */
public final class DBV3BackupReader implements BackupReader {
    @Resource
    private INotesService notesService;

    /**
     * Construct a new DBV3BackupReader.
     */
    public DBV3BackupReader() {
        super();

        Managers.getManager(IBeansManager.class).inject(this);
    }

    @Override
    public void readBackup(Object object) {
        Connection connection = (Connection) object;
        
        try {
            List<FilmImpl> films = importFilms(connection);
            List<ActorImpl> actors = importActors(connection);
            List<RealizerImpl> realizers = importRealizers(connection);
            List<KindImpl> kinds = importKinds(connection);
            List<TypeImpl> types = importTypes(connection);
            List<LanguageImpl> languages = importLanguages(connection);
            List<CountryImpl> countries = importCountries(connection);
            
            ImporterUtils.persistDataOfImport(films, actors, realizers, kinds, types, languages, countries, null, null);
        } catch (DatabaseException e) {
            Managers.getManager(ILoggingManager.class).getLogger(getClass()).error("Unable to import data", e);
        }

    }

    @Override
    public void persistTheData() {
    }

    /**
     * Import the films.
     *
     * @param connection The connection.
     * @return The imported films.
     * @throws DatabaseException Thrown when an error occurs during the database data getting process.
     */
    private List<FilmImpl> importFilms(Connection connection) throws DatabaseException {
        List<FilmImpl> films = new ArrayList<FilmImpl>(25);

        Statement statement = null;
        ResultSet result = null;
        try {
            statement = connection.createStatement();
            result = statement.executeQuery("SELECT * FROM film");

            while (result.next()) {
                FilmImpl film = new FilmImpl();

                film.getTemporaryContext().setId(result.getInt("ID"));
                film.setTitle(result.getString("nom"));

                List<Integer> kinds = new ArrayList<Integer>(1);

                kinds.add(result.getInt("genre"));

                film.getTemporaryContext().setKinds(kinds);

                film.getTemporaryContext().setType(result.getInt("type"));
                film.setYear(result.getInt("annee"));
                film.setDuration(result.getInt("duree"));
                film.getTemporaryContext().setRealizer(result.getInt("realisateur"));
                film.getTemporaryContext().setLanguage(result.getInt("langue"));
                film.setNote(notesService.getDefaultNote());

                PreparedStatement statement2 = null;
                ResultSet resultActors = null;
                try {
                    statement2 = connection.prepareStatement("SELECT * FROM film_acteur WHERE ID_film = ?");

                    statement2.setInt(1, film.getTemporaryContext().getId());

                    resultActors = statement2.executeQuery();

                    List<Integer> actors = new ArrayList<Integer>(10);

                    while (resultActors.next()) {
                        actors.add(resultActors.getInt("ID_acteur"));
                    }

                    film.getTemporaryContext().setActors(actors);
                } catch (SQLException e) {
                    throw new DatabaseException(e);
                } finally {
                    if (resultActors != null) {
                        try {
                            resultActors.close();
                        } catch (SQLException e) {
                            throw new DatabaseException(e);
                        }
                    }

                    if (statement2 != null) {
                        try {
                            statement2.close();
                        } catch (SQLException e) {
                            throw new DatabaseException(e);
                        }
                    }
                }

                films.add(film);
            }
        } catch (SQLException e) {
            throw new DatabaseException(e);
        } finally {
            if (result != null) {
                try {
                    result.close();
                } catch (SQLException e) {
                    throw new DatabaseException(e);
                }
            }

            if (statement != null) {
                try {
                    statement.close();
                } catch (SQLException e) {
                    throw new DatabaseException(e);
                }
            }
        }

        return films;
    }

    /**
     * Import the actors.
     *
     * @param connection The connection.
     * @return The imported actors.
     * @throws DatabaseException Thrown when an error occurs during the database data getting process.
     */
    private List<ActorImpl> importActors(Connection connection) throws DatabaseException {
        List<ActorImpl> actors = new ArrayList<ActorImpl>(50);

        Statement statement = null;
        ResultSet result = null;
        try {
            statement = connection.createStatement();
            result = statement.executeQuery("SELECT * FROM acteur");

            while (result.next()) {
                ActorImpl actor = new ActorImpl();

                actor.getTemporaryContext().setId(result.getInt("ID"));
                actor.setName(result.getString("nom"));
                actor.setFirstName(result.getString("prenom"));
                actor.getTemporaryContext().setCountry(result.getInt("pays"));
                actor.setNote(notesService.getDefaultNote());

                actors.add(actor);
            }
        } catch (SQLException e) {
            throw new DatabaseException(e);
        } finally {
            if (result != null) {
                try {
                    result.close();
                } catch (SQLException e) {
                    throw new DatabaseException(e);
                }
            }

            if (statement != null) {
                try {
                    statement.close();
                } catch (SQLException e) {
                    throw new DatabaseException(e);
                }
            }
        }

        return actors;
    }

    /**
     * Import the realizers.
     *
     * @param connection The connection.
     * @return The imported realizers.
     * @throws DatabaseException Thrown when an error occurs during the database data getting process.
     */
    private List<RealizerImpl> importRealizers(Connection connection) throws DatabaseException {
        List<RealizerImpl> realizers = new ArrayList<RealizerImpl>(25);

        Statement statement = null;
        ResultSet result = null;
        try {
            statement = connection.createStatement();
            result = statement.executeQuery("SELECT * FROM realisateur");

            while (result.next()) {
                RealizerImpl realizer = new RealizerImpl();

                realizer.getTemporaryContext().setId(result.getInt("ID"));
                realizer.setName(result.getString("nom"));
                realizer.setFirstName(result.getString("prenom"));
                realizer.getTemporaryContext().setCountry(result.getInt("pays"));
                realizer.setNote(notesService.getDefaultNote());

                realizers.add(realizer);
            }
        } catch (SQLException e) {
            throw new DatabaseException(e);
        } finally {
            if (result != null) {
                try {
                    result.close();
                } catch (SQLException e) {
                    throw new DatabaseException(e);
                }
            }

            if (statement != null) {
                try {
                    statement.close();
                } catch (SQLException e) {
                    throw new DatabaseException(e);
                }
            }
        }

        return realizers;
    }

    /**
     * Import the kinds.
     *
     * @param connection The connection.
     * @return The imported kinds.
     * @throws DatabaseException Thrown when an error occurs during the database data getting process.
     */
    private static List<KindImpl> importKinds(Connection connection) throws DatabaseException {
        List<KindImpl> kinds = new ArrayList<KindImpl>(10);

        Statement statement = null;
        ResultSet result = null;
        try {
            statement = connection.createStatement();
            result = statement.executeQuery("SELECT * FROM genre");

            while (result.next()) {
                KindImpl kind = new KindImpl();

                kind.getTemporaryContext().setId(result.getInt("ID"));
                kind.setName(result.getString("nom"));

                kinds.add(kind);
            }
        } catch (SQLException e) {
            throw new DatabaseException(e);
        } finally {
            if (result != null) {
                try {
                    result.close();
                } catch (SQLException e) {
                    throw new DatabaseException(e);
                }
            }

            if (statement != null) {
                try {
                    statement.close();
                } catch (SQLException e) {
                    throw new DatabaseException(e);
                }
            }
        }

        return kinds;
    }

    /**
     * Import the types.
     *
     * @param connection The connection.
     * @return The imported types.
     * @throws DatabaseException Thrown when an error occurs during the database data getting process.
     */
    private static List<TypeImpl> importTypes(Connection connection) throws DatabaseException {
        List<TypeImpl> types = new ArrayList<TypeImpl>(10);

        Statement statement = null;
        ResultSet result = null;
        try {
            statement = connection.createStatement();
            result = statement.executeQuery("SELECT * FROM type");

            while (result.next()) {
                TypeImpl type = new TypeImpl();

                type.getTemporaryContext().setId(result.getInt("ID"));
                type.setName(result.getString("nom"));

                types.add(type);
            }
        } catch (SQLException e) {
            throw new DatabaseException(e);
        } finally {
            if (result != null) {
                try {
                    result.close();
                } catch (SQLException e) {
                    throw new DatabaseException(e);
                }
            }

            if (statement != null) {
                try {
                    statement.close();
                } catch (SQLException e) {
                    throw new DatabaseException(e);
                }
            }
        }

        return types;
    }

    /**
     * Import the languages.
     *
     * @param connection The connection.
     * @return The imported languages.
     * @throws DatabaseException Thrown when an error occurs during the database data getting process.
     */
    private static List<LanguageImpl> importLanguages(Connection connection) throws DatabaseException {
        List<LanguageImpl> languages = new ArrayList<LanguageImpl>(10);

        Statement statement = null;
        ResultSet result = null;
        try {
            statement = connection.createStatement();
            result = statement.executeQuery("SELECT * FROM langue");

            while (result.next()) {
                LanguageImpl language = new LanguageImpl();

                language.getTemporaryContext().setId(result.getInt("ID"));
                language.setName(result.getString("nom"));

                languages.add(language);
            }
        } catch (SQLException e) {
            throw new DatabaseException(e);
        } finally {
            if (result != null) {
                try {
                    result.close();
                } catch (SQLException e) {
                    throw new DatabaseException(e);
                }
            }

            if (statement != null) {
                try {
                    statement.close();
                } catch (SQLException e) {
                    throw new DatabaseException(e);
                }
            }
        }

        return languages;
    }

    /**
     * Import the countries.
     *
     * @param connection The connection.
     * @return The imported countries.
     * @throws DatabaseException Thrown when an error occurs during the database data getting process.
     */
    private static List<CountryImpl> importCountries(Connection connection) throws DatabaseException {
        List<CountryImpl> countries = new ArrayList<CountryImpl>(20);

        Statement statement = null;
        ResultSet result = null;
        try {
            statement = connection.createStatement();
            result = statement.executeQuery("SELECT * FROM pays");

            while (result.next()) {
                CountryImpl country = new CountryImpl();

                country.getTemporaryContext().setId(result.getInt("ID"));
                country.setName(result.getString("nom"));

                countries.add(country);
            }
        } catch (SQLException e) {
            throw new DatabaseException(e);
        } finally {
            if (result != null) {
                try {
                    result.close();
                } catch (SQLException e) {
                    throw new DatabaseException(e);
                }
            }

            if (statement != null) {
                try {
                    statement.close();
                } catch (SQLException e) {
                    throw new DatabaseException(e);
                }
            }
        }

        return countries;
    }
}