package org.jtheque.films.services.impl.utils.file.restore;

/*
 * This file is part of JTheque.
 *
 * JTheque is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License.
 *
 * JTheque is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JTheque.  If not, see <http://www.gnu.org/licenses/>.
 */

import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.xpath.XPath;
import org.jtheque.core.managers.Managers;
import org.jtheque.core.managers.beans.IBeansManager;
import org.jtheque.core.managers.log.ILoggingManager;
import org.jtheque.core.managers.file.able.BackupReader;
import org.jtheque.core.utils.db.DaoNotes;
import org.jtheque.core.utils.db.DaoNotes.NoteType;
import org.jtheque.utils.bean.IntDate;
import org.jtheque.films.persistence.od.ActorImpl;
import org.jtheque.films.persistence.od.FilmImpl;
import org.jtheque.films.persistence.od.RealizerImpl;
import org.jtheque.films.services.able.IActorService;
import org.jtheque.films.services.able.IFilmsService;
import org.jtheque.films.services.able.IRealizersService;
import org.jtheque.films.services.impl.utils.DataUtils;
import org.jtheque.primary.od.*;
import org.jtheque.primary.services.able.*;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.Set;

/**
 * A reader for JTD Backup.
 *
 * @author Baptiste Wicht
 */
public final class XMLBackupReader implements BackupReader {
    private final Collection<LanguageImpl> languages = new ArrayList<LanguageImpl>(10);
    private final Collection<CountryImpl> countries = new ArrayList<CountryImpl>(10);
    private final Collection<KindImpl> kinds = new ArrayList<KindImpl>(10);
    private final Collection<TypeImpl> types = new ArrayList<TypeImpl>(10);
    private final Collection<LendingImpl> lendings = new ArrayList<LendingImpl>(10);
    private final Collection<BorrowerImpl> borrowers = new ArrayList<BorrowerImpl>(10);
    private final Collection<RealizerImpl> realizers = new ArrayList<RealizerImpl>(20);
    private final Collection<ActorImpl> actors = new ArrayList<ActorImpl>(50);
    private final Collection<FilmImpl> films = new ArrayList<FilmImpl>(25);

    @Resource
    private ICountriesService countriesService;

    @Resource
    private ILendingsService lendingsService;

    @Resource
    private ITypesService typesService;

    @Resource
    private ILanguagesService languagesService;

    @Resource
    private IBorrowersService borrowersService;

    @Resource
    private IFilmsService filmsService;

    @Resource
    private IActorService actorService;

    @Resource
    private IRealizersService realizersService;

    @Resource
    private IKindsService kindsService;

    private static final DaoNotes daoNotes = DaoNotes.getINSTANCE();

    /**
     * Construct a new XMLBackupReader.
     */
    public XMLBackupReader() {
        super();

        Managers.getManager(IBeansManager.class).inject(this);
    }

    @Override
    public void readBackup(Object object) {
        Element root = (Element) object;

        try {
            readLanguages(root);
            readCountries(root);
            readKinds(root);
            readTypes(root);
            readBorrowers(root);
            readRealizers(root);
            readActors(root);
            readFilms(root);
            readLendings(root);

            for (FilmImpl film : films) {
                film.setTheLending(DataUtils.getLendingForFilm(film, lendings));
            }
        } catch (JDOMException e) {
            Managers.getManager(ILoggingManager.class).getLogger(getClass()).exception(e);
        }
    }

    /**
     * Read all the languages in the root element.
     *
     * @param root The root element.
     * @throws JDOMException This exception is throwed when an error occurs during the xml reading process.
     */
    private void readLanguages(Element root) throws JDOMException {
        XPath xpa = XPath.newInstance("//languages/language");

        for (Object languageElement : xpa.selectNodes(root)) {
            LanguageImpl language = new LanguageImpl();
            language.getTemporaryContext().setId(Integer.parseInt(XPath.newInstance("./id").valueOf(languageElement)));
            language.setName(XPath.newInstance("./name").valueOf(languageElement));

            languages.add(language);

            languagesService.create(language);
        }
    }

    /**
     * Read all the countries in the root element.
     *
     * @param root The root element.
     * @throws JDOMException This exception is throwed when an error occurs during the xml reading process.
     */
    private void readCountries(Element root) throws JDOMException {
        XPath xpa = XPath.newInstance("//countries/country");

        for (Object countryElement : xpa.selectNodes(root)) {
            CountryImpl country = new CountryImpl();
            country.getTemporaryContext().setId(Integer.parseInt(XPath.newInstance("./id").valueOf(countryElement)));
            country.setName(XPath.newInstance("./name").valueOf(countryElement));

            countries.add(country);

            countriesService.create(country);
        }
    }

    /**
     * Read all the kinds in the root element.
     *
     * @param root The root element.
     * @throws JDOMException This exception is throwed when an error occurs during the xml reading process.
     */
    private void readKinds(Element root) throws JDOMException {
        XPath xpa = XPath.newInstance("//kinds/kinds");

        for (Object kindElement : xpa.selectNodes(root)) {
            KindImpl kind = new KindImpl();
            kind.getTemporaryContext().setId(Integer.parseInt(XPath.newInstance("./id").valueOf(kindElement)));
            kind.setName(XPath.newInstance("./name").valueOf(kindElement));

            kinds.add(kind);

            kindsService.create(kind);
        }
    }

    /**
     * Read all the types in the root element.
     *
     * @param root The root element.
     * @throws JDOMException This exception is throwed when an error occurs during the xml reading process.
     */
    private void readTypes(Element root) throws JDOMException {
        XPath xpa = XPath.newInstance("//types/type");

        for (Object typeElement : xpa.selectNodes(root)) {
            TypeImpl type = new TypeImpl();
            type.getTemporaryContext().setId(Integer.parseInt(XPath.newInstance("./id").valueOf(typeElement)));
            type.setName(XPath.newInstance("./id").valueOf(typeElement));

            types.add(type);

            typesService.create(type);
        }
    }

    /**
     * Read all the borrowers in the root element.
     *
     * @param root The root element.
     * @throws JDOMException This exception is throwed when an error occurs during the xml reading process.
     */
    private void readBorrowers(Element root) throws JDOMException {
        XPath xpa = XPath.newInstance("//borrowers/borrower");

        for (Object borrowerElement : xpa.selectNodes(root)) {
            BorrowerImpl borrower = new BorrowerImpl();
            borrower.getTemporaryContext().setId(Integer.parseInt(XPath.newInstance("./id").valueOf(borrowerElement)));
            borrower.setName(XPath.newInstance("./name").valueOf(borrowerElement));
            borrower.setFirstName(XPath.newInstance("./firstname").valueOf(borrowerElement));
            borrower.setEmail(XPath.newInstance("./email").valueOf(borrowerElement));

            borrowers.add(borrower);

            borrowersService.create(borrower);
        }
    }

    /**
     * Read all the realizers in the root element.
     *
     * @param root The root element.
     * @throws JDOMException This exception is throwed when an error occurs during the xml reading process.
     */
    private void readRealizers(Element root) throws JDOMException {
        XPath xpa = XPath.newInstance("//realizers/realizer");

        for (Object realizerElement : xpa.selectNodes(root)) {
            RealizerImpl realizer = new RealizerImpl();
            realizer.getTemporaryContext().setId(Integer.parseInt(XPath.newInstance("./id").valueOf(realizerElement)));
            realizer.setName(XPath.newInstance("./name").valueOf(realizerElement));
            realizer.setFirstName(XPath.newInstance("./firstname").valueOf(realizerElement));
            realizer.setTheCountry(DataUtils.getDataByTemporaryId(
                    Integer.parseInt(XPath.newInstance("./country").valueOf(realizerElement)), countries));
            realizer.setNote(daoNotes.getNote(NoteType.getEnum(Integer.parseInt(XPath.newInstance("./note").valueOf(realizerElement)))));

            realizers.add(realizer);

            realizersService.create(realizer);
        }
    }

    /**
     * Read all the actors in the root element.
     *
     * @param root The root element.
     * @throws JDOMException This exception is throwed when an error occurs during the xml reading process.
     */
    private void readActors(Element root) throws JDOMException {
        XPath xpa = XPath.newInstance("//actors/actor");

        for (Object actorElement : xpa.selectNodes(root)) {
            ActorImpl actor = new ActorImpl();
            actor.getTemporaryContext().setId(Integer.parseInt(XPath.newInstance("./id").valueOf(actorElement)));
            actor.setName(XPath.newInstance("./name").valueOf(actorElement));
            actor.setFirstName(XPath.newInstance("./firstname").valueOf(actorElement));
            actor.setTheCountry(DataUtils.getDataByTemporaryId(
                    Integer.parseInt(XPath.newInstance("./country").valueOf(actorElement)), countries));
            actor.setNote(daoNotes.getNote(NoteType.getEnum(Integer.parseInt(XPath.newInstance("./note").valueOf(actorElement)))));

            actors.add(actor);

            actorService.create(actor);
        }
    }

    /**
     * Read all the films in the root element.
     *
     * @param root The root element.
     * @throws JDOMException         This exception is throwed when an error occurs during the xml reading process.
     * @throws NumberFormatException This exception is throwed when an error occurs during the parse process.
     */
    private void readFilms(Element root) throws NumberFormatException, JDOMException {
        XPath xpa = XPath.newInstance("//films/film");

        for (Object filmElement : xpa.selectNodes(root)) {
            FilmImpl film = new FilmImpl();
            film.getTemporaryContext().setId(Integer.parseInt(XPath.newInstance("./id").valueOf(filmElement)));
            film.setTitle(XPath.newInstance("./title").valueOf(filmElement));
            film.setTheRealizer(DataUtils.getDataByTemporaryId(
                    Integer.parseInt(XPath.newInstance("./realizer").valueOf(filmElement)), realizers));
            film.setTheLanguage(DataUtils.getDataByTemporaryId(
                    Integer.parseInt(XPath.newInstance("./language").valueOf(filmElement)), languages));
            film.setTheType(DataUtils.getDataByTemporaryId(
                    Integer.parseInt(XPath.newInstance("./type").valueOf(filmElement)), types));
            film.setYear(Integer.parseInt(XPath.newInstance("./year").valueOf(filmElement)));
            film.setDuration(Integer.parseInt(XPath.newInstance("./duration").valueOf(filmElement)));
            film.setNote(daoNotes.getNote(NoteType.getEnum(Integer.parseInt(XPath.newInstance("./note").valueOf(filmElement)))));
            film.setResume(XPath.newInstance("./resume").valueOf(filmElement));
            film.setComment(XPath.newInstance("./comment").valueOf(filmElement));

            XPath xpa2 = XPath.newInstance(".//actors/actor");

            Set<ActorImpl> actorsFilm = new HashSet<ActorImpl>(12);

            for (Object actorElement : xpa2.selectNodes(filmElement)) {
                ActorImpl actor = DataUtils.getDataByTemporaryId(
                        Integer.parseInt(XPath.newInstance(".").valueOf(actorElement)), actors);

                actorsFilm.add(actor);
            }

            film.setActors(actorsFilm);

            xpa2 = XPath.newInstance(".//kinds/kind");

            Set<KindImpl> kindsFilm = new HashSet<KindImpl>(5);

            for (Object kindElement : xpa2.selectNodes(filmElement)) {
                KindImpl kind = DataUtils.getDataByTemporaryId(
                        Integer.parseInt(XPath.newInstance(".").valueOf(kindElement)), kinds);

                kindsFilm.add(kind);
            }

            film.setKinds(kindsFilm);

            films.add(film);

            filmsService.create(film);
        }
    }

    /**
     * Read all the lendings in the root element.
     *
     * @param root The root element.
     * @throws JDOMException This exception is throwed when an error occurs during the xml reading process.
     */
    private void readLendings(Element root) throws JDOMException {
        XPath xpa = XPath.newInstance("//lendings/lending");

        for (Object lendingElement : xpa.selectNodes(root)) {
            LendingImpl lending = new LendingImpl();
            lending.getTemporaryContext().setId(Integer.parseInt(XPath.newInstance("./id").valueOf(lendingElement)));
            lending.setDate(new IntDate(Integer.parseInt(XPath.newInstance("./date").valueOf(lendingElement))));
            lending.setTheBorrower(DataUtils.getDataByTemporaryId(
                    Integer.parseInt(XPath.newInstance("./borrower").valueOf(lendingElement)), borrowers));
            lending.setTheOther(DataUtils.getDataByTemporaryId(
                    Integer.parseInt(XPath.newInstance("./film").valueOf(lendingElement)), films).getId());

            lendings.add(lending);

            lendingsService.create(lending);
        }
    }

    @Override
    public void persistTheData() {
    }
}