package org.jtheque.films.view.impl.frames;

/*
 * This file is part of JTheque.
 *
 * JTheque is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License.
 *
 * JTheque is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JTheque.  If not, see <http://www.gnu.org/licenses/>.
 */

import org.jdesktop.swingx.JXRadioGroup;
import org.jtheque.core.managers.error.JThequeError;
import org.jtheque.core.utils.ui.ValidationUtils;
import org.jtheque.core.utils.ui.PanelBuilder;
import org.jtheque.core.managers.view.impl.components.panel.FileChooserPanel;
import org.jtheque.core.managers.view.impl.frame.abstraction.SwingDialogView;
import org.jtheque.core.managers.view.impl.actions.JThequeAction;
import org.jtheque.utils.ui.SwingUtils;
import org.jtheque.films.utils.Constants.Site;
import org.jtheque.films.view.able.IAutoImportView;
import org.jtheque.films.view.impl.models.AutoImportModel;
import org.jtheque.films.view.impl.models.combo.SitesComboBoxModel;

import javax.swing.DefaultListModel;
import javax.swing.JList;
import javax.swing.ListSelectionModel;
import javax.annotation.PostConstruct;

import java.awt.Color;
import java.awt.Container;
import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.util.List;

/**
 * View for the auto-import function.
 *
 * @author Baptiste Wicht
 */
public final class AutoImportVIew extends SwingDialogView implements IAutoImportView {
    private static final long serialVersionUID = 4633039680922071605L;

    private FileChooserPanel directoryChooser;
    private JList listTitles;
    private JXRadioGroup<String> fileMode;
    private JXRadioGroup<String> webMode;

    private SitesComboBoxModel sitesModel;
    private DefaultListModel modelListTitles;

    private int phase = PHASE_1;
    
    private JThequeAction searchTitlesAction;
    private JThequeAction deleteTitleAction;
    private JThequeAction addTitleAction;
    private JThequeAction validateAction;
    private JThequeAction cancelAction;

    /**
     * Construct a new JFrameAutoImport.
     * 
     * @param parent The parent frame. 
     */
    public AutoImportVIew(Frame parent) {
        super(parent);

        setModel(new AutoImportModel());
    }

    @Override
    @PostConstruct
    public void build() {
        setTitleKey("auto.import.view.title");
        setContentPane(buildContentPane());
        setResizable(false);
        pack();

        SwingUtils.centerFrame(this);
    }

    /**
     * Init the graphics user interface.
     *
     * @return The contentpane.
     */
    private Container buildContentPane() {
        PanelBuilder builder = new PanelBuilder();

        directoryChooser = new FileChooserPanel();
        directoryChooser.setDirectoriesOnly(true);
        directoryChooser.setTextKey("auto.import.file.folder");

        builder.add(directoryChooser, builder.gbcSet(0, 0, GridBagConstraints.HORIZONTAL, 2, 1));

        builder.addI18nLabel("auto.import.file.mode", builder.gbcSet(0, 1));

        String[] values = {getMessage("auto.import.file.mode.file"), getMessage("auto.import.file.mode.directory")};
        
        fileMode = new JXRadioGroup<String>(values);
        fileMode.setBackground(Color.white);
        fileMode.setSelectedValue(values[0]);

        builder.add(fileMode, builder.gbcSet(1, 1));

        builder.addButtonBar(builder.gbcSet(0, 2, GridBagConstraints.HORIZONTAL, 2, 1), searchTitlesAction);
        
        modelListTitles = new DefaultListModel();

        listTitles = new JList(modelListTitles);
        listTitles.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        listTitles.setVisibleRowCount(5);
        
        builder.addScrolled(listTitles, builder.gbcSet(0, 3, GridBagConstraints.HORIZONTAL, 2, 1));

        builder.addButtonBar(builder.gbcSet(0, 4, GridBagConstraints.HORIZONTAL, 2, 1), 
                deleteTitleAction, addTitleAction);
        
        builder.addI18nLabel("auto.import.web.mode", builder.gbcSet(0, 5));

        values = new String[]{getMessage("auto.import.web.mode.empty"), getMessage("auto.import.web.mode.fill")};
        
        webMode = new JXRadioGroup<String>(values);
        webMode.setBackground(Color.white);
        webMode.setSelectedValue(values[0]);

        builder.add(webMode, builder.gbcSet(1, 5));

        builder.addI18nLabel("auto.import.web.site", builder.gbcSet(0, 6));

        sitesModel = new SitesComboBoxModel();

        builder.addComboBox(sitesModel, builder.gbcSet(1, 6));

        builder.addButtonBar(builder.gbcSet(0, 7, GridBagConstraints.HORIZONTAL, 2, 1), 
                validateAction, cancelAction);
        
        return builder.getPanel();
    }

    @Override
    public String getFolderPath() {
        return directoryChooser.getFilePath();
    }

    @Override
    public String getSelectedTitle() {
        return getModel().getTitles().get(listTitles.getSelectedIndex());
    }

    @Override
    public DefaultListModel getModelTitles() {
        return modelListTitles;
    }

    @Override
    public AutoImportModel getModel() {
        return (AutoImportModel) super.getModel();
    }

    @Override
    public boolean isFileMode() {
        return fileMode.getSelectedValue().equals(getMessage("auto.import.file.mode.file"));
    }

    @Override
    public boolean isWebMode() {
        return webMode.getSelectedValue().equals(getMessage("auto.import.web.mode.fill"));
    }

    @Override
    public Site getSelectedSite() {
        return sitesModel.getSelectedSite();
    }

    @Override
    protected void validate(List<JThequeError> errors) {
        if (PHASE_1 == phase) {
            ValidationUtils.rejectIfEmpty(directoryChooser.getFilePath(), "auto.import.file.folder", errors);
        } else if (PHASE_2 == phase) {
            ValidationUtils.rejectIfEmpty(getModel().getTitles(), "auto.import.view.title", errors);
            ValidationUtils.rejectIfNothingSelected(sitesModel, "auto.import.web.site", errors);
        }
    }

    @Override
    public boolean validateContent(int phase) {
        this.phase = phase;

        return validateContent();
    }

    public void setSearchTitlesAction(JThequeAction searchTitlesAction) {
        this.searchTitlesAction = searchTitlesAction;
    }

    public void setDeleteTitleAction(JThequeAction deleteTitleAction) {
        this.deleteTitleAction = deleteTitleAction;
    }

    public void setAddTitleAction(JThequeAction addTitleAction) {
        this.addTitleAction = addTitleAction;
    }

    public void setValidateAction(JThequeAction validateAction) {
        this.validateAction = validateAction;
    }

    public void setCancelAction(JThequeAction cancelAction) {
        this.cancelAction = cancelAction;
    }
}