package org.jtheque.films.view.impl.panels;

/*
 * This file is part of JTheque.
 *
 * JTheque is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License.
 *
 * JTheque is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JTheque.  If not, see <http://www.gnu.org/licenses/>.
 */

import org.jdesktop.swingx.JXPanel;
import org.jtheque.core.managers.error.JThequeError;
import org.jtheque.core.managers.view.impl.actions.JThequeAction;
import org.jtheque.core.utils.ui.PanelBuilder;
import org.jtheque.films.persistence.od.RealizerImpl;
import org.jtheque.films.persistence.od.abstraction.Film;
import org.jtheque.core.utils.ui.constraints.ConstraintManager;
import org.jtheque.films.services.able.IRealizersService;
import org.jtheque.films.utils.Constants.Properties;
import org.jtheque.films.view.able.IInfosFilmView;
import org.jtheque.films.view.impl.fb.FilmFormBean;
import org.jtheque.primary.od.LanguageImpl;
import org.jtheque.primary.od.SagaImpl;
import org.jtheque.primary.services.able.ILanguagesService;
import org.jtheque.primary.services.able.ISagasService;
import org.jtheque.primary.view.impl.listeners.CurrentObjectListener;
import org.jtheque.primary.view.impl.listeners.ObjectChangedEvent;
import org.jtheque.primary.view.impl.models.DataContainerCachedComboBoxModel;
import org.jtheque.utils.ui.GridBagUtils;

import javax.annotation.Resource;
import javax.annotation.PostConstruct;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFormattedTextField;
import javax.swing.JTextArea;
import javax.swing.JComponent;
import javax.swing.text.DefaultFormatter;
import java.text.NumberFormat;
import java.util.List;

/**
 * Cette classe reprsente le panel prsentant les options gnrales du film.
 *
 * @author Baptiste Wicht
 */
public final class JPanelInfosFilm extends JXPanel implements CurrentObjectListener, IInfosFilmView {
    private static final long serialVersionUID = 7846094711921875146L;

    private DataContainerCachedComboBoxModel<RealizerImpl> modelRealisateur;
    private DataContainerCachedComboBoxModel<LanguageImpl> modelLangue;
    private DataContainerCachedComboBoxModel<SagaImpl> modelSaga;

    private JComboBox comboRealisateur;
    private JComboBox comboLangue;
    private JComboBox comboSaga;

    private JFormattedTextField fieldAnnee;
    private JFormattedTextField fieldDuree;
    private JTextArea areaResume;

    private JButton buttonAddLangue;
    private JButton buttonAddSaga;
    
    private JThequeAction newLanguageAction;
    private JThequeAction newSagaAction;

    @Resource
    private ILanguagesService languagesService;

    @Resource
    private IRealizersService realizersService;

    @Resource
    private ISagasService sagasService;

    /**
     * Cette mthode initialise le panel.
     */
    @PostConstruct
    private void build() {
        PanelBuilder builder = new PanelBuilder(this);

        builder.addI18nLabel(Properties.Film.REALIZER, builder.gbcSet(0, 0));

        modelRealisateur = new DataContainerCachedComboBoxModel<RealizerImpl>(realizersService);
        
        comboRealisateur = builder.addComboBox(modelRealisateur, builder.gbcSet(1, 0));
        comboRealisateur.setEnabled(false);

        builder.addI18nLabel(Properties.Film.YEAR, builder.gbcSet(0, 1));

        NumberFormat format = NumberFormat.getNumberInstance();
        format.setParseIntegerOnly(true);
        format.setGroupingUsed(false);

        fieldAnnee = new JFormattedTextField(format);
        fieldAnnee.setColumns(4);
        ((DefaultFormatter) fieldAnnee.getFormatter()).setAllowsInvalid(false);
        fieldAnnee.setEnabled(false);

        ConstraintManager.configure(fieldAnnee, Properties.Film.YEAR);

        builder.add(fieldAnnee, builder.gbcSet(1, 1));

        builder.addI18nLabel(Properties.Film.DURATION, builder.gbcSet(0, 2));
        
        fieldDuree = builder.add(new JFormattedTextField(format), builder.gbcSet(1, 2));
        fieldDuree.setColumns(4);
        fieldDuree.setEnabled(false);

        ConstraintManager.configure(fieldDuree, Properties.Film.DURATION);

        builder.addI18nLabel(Properties.Film.LANGUAGE, builder.gbcSet(0, 3));

        modelLangue = new DataContainerCachedComboBoxModel<LanguageImpl>(languagesService);
        
        comboLangue = builder.addComboBox(modelLangue, builder.gbcSet(1, 3));
        comboLangue.setEnabled(false);
        
        buttonAddLangue = builder.addButton(newLanguageAction, builder.gbcSet(2, 3));
        buttonAddLangue.setEnabled(false);

        builder.addI18nLabel(Properties.Film.SAGA, builder.gbcSet(0, 4));

        modelSaga = new DataContainerCachedComboBoxModel<SagaImpl>(sagasService);
        
        comboSaga = builder.addComboBox(modelSaga, builder.gbcSet(1, 4));
        comboSaga.setEnabled(false);
        
        buttonAddSaga = builder.addButton(newSagaAction, builder.gbcSet(2, 4));
        buttonAddSaga.setEnabled(false);

        builder.addI18nLabel(Properties.Film.RESUME, builder.gbcSet(0, 5));

        areaResume = new JTextArea();
        areaResume.setEnabled(false);
        areaResume.setLineWrap(true);
        areaResume.setWrapStyleWord(true);
        areaResume.setAutoscrolls(true);
        
        builder.addScrolled(areaResume, builder.gbcSet(0, 6, GridBagUtils.BOTH, GridBagUtils.ABOVE_BASELINE_LEADING, 0, 0, 1.0, 1.0));
    }

    @Override
    public void objectChanged(ObjectChangedEvent event) {
        Film film = (Film) event.getObject();

        if (film.getTheRealizer() != null) {
            modelRealisateur.setSelectedItem(film.getTheRealizer());
        }

        if (film.getTheLanguage() != null) {
            modelLangue.setSelectedItem(film.getTheLanguage());
        }

        if (film.getTheSaga() != null) {
            modelSaga.setSelectedItem(film.getTheSaga());
        }

        fieldAnnee.setText(Integer.toString(film.getYear()));
        fieldDuree.setText(Integer.toString(film.getDuration()));
        areaResume.setText(film.getResume());
    }

    /**
     * Fill the formbean.
     *
     * @param fb The formbean to fill.
     */
    @Override
    public void fillFilm(FilmFormBean fb) {
        fb.setRealizer(modelRealisateur.getSelectedData());
        fb.setLanguage(modelLangue.getSelectedData());
        fb.setYear(Integer.parseInt(fieldAnnee.getText()));
        fb.setDuration(Integer.parseInt(fieldDuree.getText()));
        fb.setResume(areaResume.getText());
        fb.setSaga(modelSaga.getSelectedData());
    }

    @Override
    public void setEnabled(boolean enabled) {
        comboRealisateur.setEnabled(enabled);
        comboLangue.setEnabled(enabled);
        comboSaga.setEnabled(enabled);
        fieldAnnee.setEnabled(enabled);
        fieldDuree.setEnabled(enabled);
        areaResume.setEnabled(enabled);
        buttonAddLangue.setEnabled(enabled);
        buttonAddSaga.setEnabled(enabled);

        super.setEnabled(enabled);
    }

    /**
     * Validate the view.
     *
     * @param errors The errors list to fill.
     */
    @Override
    public void validate(List<JThequeError> errors) {
        ConstraintManager.validate(Properties.Film.REALIZER, fieldAnnee.getText(), errors);
        ConstraintManager.validate(Properties.Film.DURATION, fieldDuree.getText(), errors);
        ConstraintManager.validate(Properties.Film.RESUME, areaResume.getText(), errors);
        ConstraintManager.validate(Properties.Film.REALIZER, modelRealisateur.getSelectedData(), errors);
        ConstraintManager.validate(Properties.Film.SAGA, modelSaga.getSelectedData(), errors);
        ConstraintManager.validate(Properties.Film.LANGUAGE, modelLangue.getSelectedData(), errors);
    }

    public void setNewSagaAction(JThequeAction newSagaAction) {
        this.newSagaAction = newSagaAction;
    }

    public void setNewLanguageAction(JThequeAction newLanguageAction) {
        this.newLanguageAction = newLanguageAction;
    }

    @Override
    public JComponent getImpl() {
        return this;
    }
}